@@ -38,10 +38,17 @@ type TaskFlags struct {
 
 type RunFlags struct {
 	// Flags
-	Edit     bool
-	DryRun   bool
-	Describe bool
-	Silent   bool
+	Edit      bool
+	DryRun    bool
+	Describe  bool
+	ListHosts bool
+	Silent    bool
+	Confirm   bool
+	Step      bool
+	Verbose   bool
+
+	// Reports
+	Report []string
 
 	// Target
 	All     bool
@@ -52,6 +59,8 @@ type RunFlags struct {
 	Invert  bool
 	Limit   uint32
 	LimitP  uint8
+	Target  string
+	Order   string
 
 	// Config
 	KnownHostsFile string
@@ -64,25 +73,48 @@ type RunFlags struct {
 
 	// Server
 	IdentityFile string
+	User         string
 	Password     string
 
 	// Spec
-	Parallel          bool
+	Spec              string
 	AnyErrorsFatal    bool
+	MaxFailPercentage uint8
 	IgnoreErrors      bool
 	IgnoreUnreachable bool
-	OmitEmpty         bool
+	OmitEmptyRows     bool
+	OmitEmptyColumns  bool
+	Forks             uint32
+	Batch             uint32
+	BatchP            uint8
 	Output            string
+	Strategy          string
 }
 
 type SetRunFlags struct {
+	Silent            bool
+	Describe          bool
+	ListHosts         bool
+	Attach            bool
 	All               bool
 	Invert            bool
-	Parallel          bool
-	OmitEmpty         bool
+	OmitEmptyRows     bool
+	OmitEmptyColumns  bool
 	Local             bool
 	TTY               bool
 	AnyErrorsFatal    bool
 	IgnoreErrors      bool
 	IgnoreUnreachable bool
+	Order             bool
+	Report            bool
+	Batch             bool
+	BatchP            bool
+	Servers           bool
+	Tags              bool
+	Regex             bool
+	Limit             bool
+	LimitP            bool
+	Verbose           bool
+	Confirm           bool
+	Step              bool
 }
